package edu.lawrence.auction.interfaces;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import edu.lawrence.auction.interfaces.dtos.BillResponse;
import edu.lawrence.auction.interfaces.dtos.OfferResponse;
import edu.lawrence.auction.interfaces.dtos.ShippingCharge;
import edu.lawrence.auction.interfaces.dtos.ShippingConfirmation;
import edu.lawrence.auction.security.AuctionUserDetails;
import edu.lawrence.auction.services.PurchaseService;

@RestController
@RequestMapping("/purchases")
@CrossOrigin(origins = "*")
public class PurchaseController {
	private PurchaseService purchaseService;
	
    public PurchaseController(PurchaseService purchaseService) {
        this.purchaseService = purchaseService;
    }
	
    @GetMapping("/concludeauctions")
    public String concludeauctions() {
    	purchaseService.concludeAuctions();
    	return "Done.";
    }
    
	@PostMapping("/offerresponse")
	public ResponseEntity<String> offerresponse(Authentication authentication,@RequestBody OfferResponse or) {
		AuctionUserDetails details = (AuctionUserDetails) authentication.getPrincipal();
		boolean accept = false;
		try {
			accept = purchaseService.saveOfferResponse(details.getUsername(),or);
		} catch(Exception ex) {
			return ResponseEntity.status(HttpStatus.FORBIDDEN).body(ex.getMessage());
		}
		if(accept)
			return ResponseEntity.status(HttpStatus.ACCEPTED).body("Offer accepted");
		else {
			purchaseService.concludeAuctions();
			return ResponseEntity.status(HttpStatus.OK).body("Offer cancelled");
		}
	}
	
	@PostMapping("/billshipping")
    public ResponseEntity<String> billshipping(Authentication authentication,@RequestBody ShippingCharge sc) {
		AuctionUserDetails details = (AuctionUserDetails) authentication.getPrincipal();
		try {
			purchaseService.saveShipping(details.getUsername(),sc);
		} catch (Exception ex) {
			return ResponseEntity.status(HttpStatus.FORBIDDEN).body(ex.getMessage());
		}
		return ResponseEntity.status(HttpStatus.ACCEPTED).body("Shipping charge entered");
	}
	
	@PostMapping("/billresponse")
	public ResponseEntity<String> billresponse(Authentication authentication,@RequestBody BillResponse br) {
		AuctionUserDetails details = (AuctionUserDetails) authentication.getPrincipal();
		boolean accept = false;
		try {
			accept = purchaseService.saveBillResponse(details.getUsername(),br);
		} catch (Exception ex) {
			return ResponseEntity.status(HttpStatus.FORBIDDEN).body(ex.getMessage()); 
		}
		if(accept)
			return ResponseEntity.status(HttpStatus.ACCEPTED).body("Bill accepted");
		else {
			purchaseService.concludeAuctions();
			return ResponseEntity.status(HttpStatus.OK).body("Bill cancelled");
		}
	}

	@PostMapping("/confirmshipping")
	public ResponseEntity<String> confirmshipping(Authentication authentication,@RequestBody ShippingConfirmation sc) {
		AuctionUserDetails details = (AuctionUserDetails) authentication.getPrincipal();
		try {
			purchaseService.saveShipConfirm(details.getUsername(),sc);
		} catch (Exception ex) {
			return ResponseEntity.status(HttpStatus.FORBIDDEN).body(ex.getMessage());
		}
		return ResponseEntity.status(HttpStatus.ACCEPTED).body("Tracking number entered");
	}
}

