package edu.lawrence.auction.services;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import edu.lawrence.auction.entities.Auction;
import edu.lawrence.auction.entities.Bid;
import edu.lawrence.auction.entities.Tag;
import edu.lawrence.auction.entities.User;
import edu.lawrence.auction.interfaces.dtos.AuctionDTO;
import edu.lawrence.auction.interfaces.dtos.BidDTO;
import edu.lawrence.auction.repositories.AuctionRepository;
import edu.lawrence.auction.repositories.BidRepository;
import edu.lawrence.auction.repositories.TagRepository;
import edu.lawrence.auction.repositories.UserRepository;
import edu.lawrence.auction.security.WrongUserException;

@Service
public class AuctionService {
	@Autowired
	UserRepository userRepository;
	
	@Autowired
	AuctionRepository auctionRepository;
	
	@Autowired
	TagRepository tagRepository;
	
	@Autowired
	BidRepository bidRepository;
	
	public String save(AuctionDTO auction) throws WrongUserException {
		Optional<User> maybeUser = userRepository.findById(UUID.fromString(auction.getSeller()));
		if(!maybeUser.isPresent()) 
			throw new WrongUserException();
		
		User user = maybeUser.get();
		
		Auction newAuction = new Auction(auction);
		newAuction.setSeller(user);
		auctionRepository.save(newAuction);
		
		for(String t : auction.getTags()) {
			Tag newTag = new Tag();
			newTag.setAuction(newAuction);
			newTag.setTag(t);
			tagRepository.save(newTag);
		}
		return newAuction.getAuctionid().toString();
	}
	
	public Auction findById(UUID id) {
		return auctionRepository.findById(id).get();
	}
	
	public List<Auction> findActive() {
		return auctionRepository.findActive(LocalDate.now());
	}
	
	public List<Auction> findActiveByTag(String tag) {
		List<Auction> results = new ArrayList<Auction>();
		List<Tag> tags = tagRepository.findActiveByTag(LocalDate.now(), tag);
		for(Tag t : tags) {
			results.add(t.getAuction());
		}
		return results;
	}
	
	public void saveBid(UUID auctionid,BidDTO bid) throws WrongAuctionException, WrongUserException {
		Optional<Auction> maybeAuction = auctionRepository.findById(auctionid);
		if(!maybeAuction.isPresent())
			throw new WrongAuctionException();
		Auction auction = maybeAuction.get();
		
		Optional<User> maybeUser = userRepository.findById(UUID.fromString(bid.getBidder()));
		if(!maybeUser.isPresent())
			throw new WrongUserException();
		User user = maybeUser.get();
		
		Bid newBid = new Bid(bid);
		newBid.setAuction(auction);
		newBid.setBidder(user);
		bidRepository.save(newBid);
	}
}
