package edu.lawrence.auction.services;

import java.time.LocalDate;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import edu.lawrence.auction.entities.Auction;
import edu.lawrence.auction.entities.Bid;
import edu.lawrence.auction.entities.Purchase;
import edu.lawrence.auction.entities.Shipping;
import edu.lawrence.auction.interfaces.dtos.BillResponse;
import edu.lawrence.auction.interfaces.dtos.OfferResponse;
import edu.lawrence.auction.interfaces.dtos.ShippingCharge;
import edu.lawrence.auction.interfaces.dtos.ShippingConfirmation;
import edu.lawrence.auction.repositories.AuctionRepository;
import edu.lawrence.auction.repositories.BidRepository;
import edu.lawrence.auction.repositories.PurchaseRepository;
import edu.lawrence.auction.repositories.ShippingRepository;
import edu.lawrence.auction.security.WrongUserException;

@Service
public class PurchaseService {
	@Autowired
	PurchaseRepository purchaseRepository;
	
	@Autowired
	ShippingRepository shippingRepository;
	
	@Autowired
	BidRepository bidRepository;
	
	@Autowired 
	AuctionRepository auctionRepository;
	
	public boolean saveOfferResponse(String buyerID,OfferResponse or) throws Exception {
		Optional<Purchase> maybeP = purchaseRepository.findById(UUID.fromString(or.getPurchaseid()));
		if(!maybeP.isPresent())
			return false;
		Purchase p = maybeP.get();
		
		if(!p.getBid().getBidder().getUserid().toString().equals(buyerID))
			throw new WrongUserException();
		
		if(or.getAccept()) {
			Optional<Shipping> maybeS = shippingRepository.findById(or.getShippingid());
			if(!maybeS.isPresent())
				return false;
			Shipping s = maybeS.get();
			p.setShipping(s);
			p.setStatus("Confirmed");
			return true;
		} else {
			Bid b = p.getBid();
			bidRepository.delete(b);
			purchaseRepository.delete(p);
			return false;
		}
	}
    
	public void saveShipping(String sellerID,ShippingCharge sc) throws Exception {
		Optional<Purchase> maybeP = purchaseRepository.findById(UUID.fromString(sc.getPurchaseid()));
		if(maybeP.isPresent()) {
			Purchase p = maybeP.get();
			if(p.getBid().getAuction().getSeller().getUserid().toString().equals(sellerID)) {
				p.setCharged(sc.getCharge());
				p.setStatus("Charged");
			} else
				throw new WrongUserException();
		}
    }
    
	public boolean saveBillResponse(String buyerID,BillResponse br) throws Exception {
		Optional<Purchase> maybeP = purchaseRepository.findById(UUID.fromString(br.getPurchaseid()));
		if(!maybeP.isPresent())
			return false;
		Purchase p = maybeP.get();
		if(!p.getBid().getBidder().getUserid().toString().equals(buyerID))
			throw new WrongUserException();
		
		if(br.getAccepted()) {
    		p.setStatus("Paid");
    		return true;
    	} else {
    		Bid b = p.getBid();
    		bidRepository.delete(b);
			purchaseRepository.delete(p);
			return false;
    	}
    }
    
	public void saveShipConfirm(String sellerID,ShippingConfirmation sc) throws Exception {
		Optional<Purchase> maybeP = purchaseRepository.findById(UUID.fromString(sc.getPurchaseid()));
		if(maybeP.isPresent()) {
			Purchase p = maybeP.get();
			if(p.getBid().getAuction().getSeller().getUserid().toString().equals(sellerID)) {
				p.setTracking(sc.getTracking());
				p.setStatus("Shipped");
			} else
				throw new WrongUserException();
		}
    }
    
	public void concludeAuctions() {
    	// Find all of the completed auctions for which there is no purchase
    	List<Auction> auctions =  auctionRepository.findIncomplete(LocalDate.now());

    	for(Auction a : auctions) {
    		// Find the winning bid
    		List<Bid> bids = a.getBids();
    		if(bids.size() > 0) {
	    		Bid winner = bids.get(0);
	    		for(Bid b : bids) {
	    			if(b.getBid() > winner.getBid())
	    				winner = b;
	    		}
	    		// Create a Purchase
	    		Purchase p = new Purchase();
	    		p.setBid(winner);
	    		p.setStatus("Won_bid");
	    		purchaseRepository.save(p);
    		}
    		// Mark the auction as completed
	    	a.setCompleted(true);
    		
    	}
    }
}
